@@ -6,13 +6,13 @@ import os
 import string
 import codecs
 import logging
-from xfdfgen import Xfdf
 from io import BytesIO
+from xfdfgen import Xfdf
 import pikepdf
 import img2pdf
 from pikepdf import Pdf
 from PIL import Image
-from docassemble.base.error import DAError
+from docassemble.base.error import DAError, DAException
 from docassemble.base.pdfa import pdf_to_pdfa
 from docassemble.base.logger import logmessage
 from docassemble.base.functions import word
@@ -71,7 +71,7 @@ def recursively_add_fields(fields, id_to_page, outfields, prefix='', parent_ft=N
             name, value, rect, page, field_type = field.get('T'), field.get('V'), field.get('Rect'), field.get('P'), field.get('FT')
             if field_type is None:
                 widget_type = str(field.get("Type"))
-                if widget_type == "/'Annot'" or widget_type == "/Annot":
+                if widget_type in ("/'Annot'", "/Annot"):
                     field_type = parent_ft
         except:
             logmessage("Skipping field " + repr(field))
@@ -299,6 +299,7 @@ def fill_template(template, data_strings=None, data_names=None, hidden=None, rea
                         annot.DV = the_string
         if len(images) == 0:
             pdf.save(pdf_file.name)
+            pdf.close()
     if len(images) > 0:
         fields = {}
         for field, default, pageno, rect, field_type, export_value in the_fields:
@@ -344,10 +345,11 @@ def fill_template(template, data_strings=None, data_names=None, hidden=None, rea
             for item in image_todo:
                 xone, yone, xtwo, ytwo = fields[item['field']]['rect']
                 logmessage("Trying to save to page " + repr(item['pageno'] - 1))
-                overlay_file = Pdf.open(item['overlay_file'])
-                overlay_page = overlay_file.pages[0]
-                pdf.pages[item['pageno'] - 1].add_overlay(overlay_page, rect=pikepdf.Rectangle(xone, yone, xtwo, ytwo))
+                with Pdf.open(item['overlay_file']) as overlay_file:
+                    overlay_page = overlay_file.pages[0]
+                    pdf.pages[item['pageno'] - 1].add_overlay(overlay_page, rect=pikepdf.Rectangle(xone, yone, xtwo, ytwo))
         pdf.save(pdf_file.name)
+        pdf.close()
     if (pdfa or not editable) and len(images) > 0:
         flatten_pdf(pdf_file.name)
     if pdfa:
@@ -480,6 +482,8 @@ def overlay_pdf(main_file, logo_file, out_file, first_page=None, last_page=None,
                 continue
         main_pdf.pages[page_no].add_overlay(logo_pdf.pages[logo_page - 1])
     main_pdf.save(out_file)
+    logo_pdf.close()
+    main_pdf.close()
 
 
 def apply_qpdf(filename):
@@ -497,6 +501,6 @@ def extract_pages(input_path, output_path, first, last):
     try:
         result = subprocess.run(subprocess_arguments, timeout=60, check=False).returncode
     except subprocess.TimeoutExpired:
-        raise Exception("call to pdftk took too long where arguments were " + " ".join(subprocess_arguments))
+        raise DAException("call to pdftk took too long where arguments were " + " ".join(subprocess_arguments))
     if result != 0:
-        raise Exception("call to pdftk failed where arguments were " + " ".join(subprocess_arguments))
+        raise DAException("call to pdftk failed where arguments were " + " ".join(subprocess_arguments))
